/*
    ettercap -- gtk interface for plugins

    Copyright (C) 2001  ALoR <alor@users.sourceforge.net>, NaGA <crwm@freemail.it>
    GTK+ 2.0 interface by daten <daten@dnetc.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*/

#include <ec_main.h>

#ifdef PERMIT_PLUGINS

#include <ctype.h>
#if defined(HAVE_SYS_POLL_H)
	#include <sys/poll.h>
#elif defined (HAVE_POLL_H)
	#include <poll.h>
#endif

#include <ec_gtk.h>
#include <ec_plugins.h>
#include <ec_buffer.h>

#define PAD_BUFFER 500        // buffered lines for plugin window

// protos....

void Ginterface_Plugins_Run(void);
void Ginterface_Plugins_PointItem(GtkWidget *widget, gpointer data);
void Ginterface_Plugins_InitList(void);
void Ginterface_Plugins_SelectItem(GtkTreeView *treeview, gpointer arg1, GtkTreeViewColumn *arg2, gpointer data);
void Ginterface_Plugins_Output(void);
gboolean Ginterface_Plugins_Output_Update(gpointer data);
void Ginterface_Plugins_ToolBar(GtkItemFactory *host_menu);
void Ginterface_Plugins_Stop(void);

void Ginterface_Plugins_Output_Callback(GtkWidget *widget, gpointer data);
void Ginterface_Plugins_Callback(gpointer data, guint action, GtkWidget *widget);

// global variables
GtkListStore *p_list = NULL;
GtkTreeIter p_iter;
GtkWidget *p_treeview = NULL;
GtkTreeSelection *p_selection = NULL;
GtkWidget *p_scrolled = NULL;

GtkWidget *o_win = NULL; /* plugin output window */
GtkTextBuffer *o_buf;

int id_watch;
extern struct plug_array *Plugins_Array;  // ec_plugins.c
short number_of_plugins;
extern int Sel_Number;
int gPlug_Base_Pointer = 0;
int gPlug_Pointer = 0;
short P_Sel_Number;

// -------------------

void Ginterface_Plugins_InitList(void)
{
   int j;
   gchar ver[5], stat[2];

   if (number_of_plugins == 0)      // no plugin... no action... ;)
   {
      Ginterface_PopUp ("NO plugin available in %s or in ./ !!\n", PLUGIN_PATH);
      return;
   }

   g_signal_handlers_block_by_func (G_OBJECT (p_selection), G_CALLBACK (Ginterface_Plugins_PointItem), NULL);

   if(p_list == NULL) {
      /* number, hostname, ip, divider, hostname, ip, status, type */
      p_list = gtk_list_store_new (5, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
      gtk_tree_view_set_model (GTK_TREE_VIEW (p_treeview), GTK_TREE_MODEL (p_list));
   } else
      gtk_list_store_clear( GTK_LIST_STORE (p_list) );

   for(j=gPlug_Base_Pointer; j<number_of_plugins; j++) 
   {
      snprintf(ver, sizeof(ver), "%.1f", Plugins_Array[j].version);
      snprintf(stat, sizeof(stat), "%c", Plugins_Array[j].status);

      gtk_list_store_append (GTK_LIST_STORE (p_list), &p_iter);
      gtk_list_store_set (GTK_LIST_STORE (p_list), &p_iter,
            0, j+1,
            1, Plugins_Array[j].name,
            2, ver,
            3, stat,
            4, Plugins_Array[j].description, -1);
   }

   g_signal_handlers_unblock_by_func (G_OBJECT (p_selection), G_CALLBACK (Ginterface_Plugins_PointItem), NULL);
}

void Ginterface_Plugins_PointItem(GtkWidget *widget, gpointer data)
{
   GtkTreeIter iter;
   GtkTreeModel *model;
   int line;

   if (number_of_plugins == 0) return; // no plugin... no action... ;)

   if (gtk_tree_selection_get_selected (GTK_TREE_SELECTION (widget), &model, &iter)) {
      gtk_tree_model_get (model, &iter, 0, &line, -1);
   } else return;

   gPlug_Pointer = line - 1;
}

void Ginterface_Plugins_SelectItem(GtkTreeView *treeview, gpointer arg1, GtkTreeViewColumn *arg2, gpointer data)
{
   DEBUG_MSG("Ginterface_Plugins_SelectItem");

   if (number_of_plugins == 0) return; // no plugin... no action... ;)

   if (Plugins_Array[gPlug_Pointer].status == 'E')
   {
      Ginterface_Plugins_Output(); /* opens a plugin output window */

      if (!Plugin_RunExt(Plugins_Array[gPlug_Pointer].name))
      {
         Ginterface_PopUp("There was an error loading this plugin !!");
         return;
      }
   }
   else  // it is a hooking plugin
   {
      Plugins_Array[gPlug_Pointer].status = (Plugins_Array[gPlug_Pointer].status == 'A') ? ' ' : 'A';
      Ginterface_Plugins_InitList();
   }
}

void Ginterface_Plugins_Output(void)
{
   GtkWidget *scrolled, *textview;

   if(o_win && !(GTK_WIDGET_VISIBLE(o_win)) ) {
      gtk_widget_show(o_win);
      return;
   } else if(o_win && GTK_WIDGET_VISIBLE(o_win) ) {
      gtk_widget_hide(o_win);
      return;
   }

   o_win = gtk_dialog_new_with_buttons ("Plugin Output", GTK_WINDOW (window), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_OK, GTK_RESPONSE_NONE, NULL);
   gtk_window_set_default_size(GTK_WINDOW (o_win), 500, 300);
   gtk_window_set_position (GTK_WINDOW (o_win), GTK_WIN_POS_CENTER);
   g_signal_connect_swapped (GTK_OBJECT (o_win), "response", G_CALLBACK (gtk_widget_hide), GTK_OBJECT (o_win));

   /* make the scrolled window */
   scrolled = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
   gtk_container_add (GTK_CONTAINER (GTK_DIALOG(o_win)->vbox), scrolled);
   gtk_widget_show (scrolled);

   /* make the Text View window */
   textview = gtk_text_view_new();
   gtk_text_view_set_editable(GTK_TEXT_VIEW (textview), FALSE);
   gtk_text_view_set_left_margin(GTK_TEXT_VIEW (textview), 10);
   gtk_text_view_set_right_margin(GTK_TEXT_VIEW (textview), 10);
   gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolled), textview);
   gtk_widget_show(textview);

   o_buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview));

   // id_watch = gtk_timeout_add(1000, Ginterface_Plugins_Output_Update, NULL);

   gtk_widget_show(o_win);
   while (gtk_events_pending ())
      gtk_main_iteration ();
}

void Ginterface_Plugins_Output_Callback (GtkWidget *widget, gpointer data) {
   gchar action = '!';

   sscanf(data, "%c", &action);

   switch (action) {
/*
      case 'D':
      case 'd':
         {
            FILE *bin_dump, *str_dump;
            int ch, ok = 0;

            bin_dump = tmpfile();
            putwin(output_window, bin_dump);
            str_dump = fopen("plugin_output_dump.log", "wb");
            rewind(bin_dump);
            fseek(bin_dump, 0x48, SEEK_CUR); // skip the header file
            while( (ch = fgetc(bin_dump)) != EOF)
            {
               if (isprint(ch))
               {
                  if (!ok && ch != 0x20) ok = 1;
                  if (ok)
                  {
                     if (ok++ == (W_MAINX2-2))
                     {
                        fputc('\n', str_dump);
                        ok = 1;
                     }
                     else
                        fputc(ch, str_dump);
                  }
               }
            }
            Ginterface_PopUp("Data dumped to : plugin_output_dump.log");
            touchwin(o_win);
            wnoutrefresh(o_win);
         }
         break;
*/
      case 'H':
      case 'h':
         {
            static char *help[] = {
               "[qQ][F10] - quit",
               "[arrows]  - scroll the output",
               "[dD]      - dump plugins output to a file",
               "[hH]      - this help screen",
               NULL};
            Ginterface_HelpWindow(help);
         }
         break;

      case 'Q':
      case 'q':
         {
            char answer;
            answer = Ginterface_PopUpQ("Do you really want to exit?");
            if ((answer == 'y') || (answer == 'Y'))
               Ginterface_WExit("They are safe!!  for now... ");
         }
         break;
   }
}


gboolean Ginterface_Plugins_Output_Update(gpointer data)
{
   int mesglen = 0;
   char message[500];

   Buffer_Get(pipe_with_plugins, &mesglen, sizeof(int));
   Buffer_Get(pipe_with_plugins, message, mesglen);

   message[mesglen] = 0;

   if (!mesglen) return(TRUE);

   gtk_text_buffer_insert_at_cursor(GTK_TEXT_BUFFER (o_buf), message, -1);

   return(TRUE);
}

void Ginterface_Plugins_Run(void)
{
   GtkCellRenderer *p_renderer;
   GtkTreeViewColumn *p_column;

   DEBUG_MSG("Ginterface_Plugins_Run");

   if (!Plugins_Array) number_of_plugins = Plugin_ExtArray();

   /* make the scrolled window */
   p_scrolled = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (p_scrolled), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);

   p_treeview = gtk_tree_view_new ();
   gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (p_treeview), FALSE);
      /* add the column */
   p_renderer = gtk_cell_renderer_text_new ();
   p_column = gtk_tree_view_column_new_with_attributes ("#", p_renderer, "text", 0, NULL);
   gtk_tree_view_column_set_sort_column_id (p_column, 0);
   gtk_tree_view_column_set_resizable (p_column, FALSE);
   gtk_tree_view_append_column (GTK_TREE_VIEW(p_treeview), p_column);
      /* add the column */
   p_renderer = gtk_cell_renderer_text_new ();
   p_column = gtk_tree_view_column_new_with_attributes ("Name", p_renderer, "text", 1, NULL);
   gtk_tree_view_column_set_sort_column_id (p_column, 1);
   gtk_tree_view_column_set_resizable (p_column, FALSE);
   gtk_tree_view_append_column (GTK_TREE_VIEW(p_treeview), p_column);
      /* add the column */
   p_renderer = gtk_cell_renderer_text_new ();
   p_column = gtk_tree_view_column_new_with_attributes ("Ver", p_renderer, "text", 2, NULL);
   gtk_tree_view_column_set_sort_column_id (p_column, 2);
   gtk_tree_view_column_set_resizable (p_column, FALSE);
   gtk_tree_view_append_column (GTK_TREE_VIEW(p_treeview), p_column);
      /* add the column */
   p_renderer = gtk_cell_renderer_text_new ();
   p_column = gtk_tree_view_column_new_with_attributes ("", p_renderer, "text", 3, NULL);
   gtk_tree_view_column_set_sort_column_id (p_column, 3);
   gtk_tree_view_column_set_resizable (p_column, FALSE);
   gtk_tree_view_append_column (GTK_TREE_VIEW(p_treeview), p_column);
      /* add the column */
   p_renderer = gtk_cell_renderer_text_new ();
   p_column = gtk_tree_view_column_new_with_attributes ("Description", p_renderer, "text", 4, NULL);
   gtk_tree_view_column_set_sort_column_id (p_column, 4);
   gtk_tree_view_column_set_resizable (p_column, FALSE);
   gtk_tree_view_append_column (GTK_TREE_VIEW(p_treeview), p_column);
      /* connect signals */
   p_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (p_treeview));
   gtk_tree_selection_set_mode (p_selection, GTK_SELECTION_SINGLE);
   g_signal_connect (G_OBJECT (p_selection), "changed", G_CALLBACK (Ginterface_Plugins_PointItem), NULL);
   g_signal_connect (G_OBJECT (p_treeview), "row_activated", G_CALLBACK (Ginterface_Plugins_SelectItem), NULL);
      /* add it and display it */
   gtk_container_add(GTK_CONTAINER (p_scrolled), p_treeview);
   gtk_widget_show(p_treeview);

   Ginterface_Plugins_InitList();

   gtk_container_add(GTK_CONTAINER (mainbox[5]), p_scrolled);
   gtk_widget_show (p_scrolled);

   id_watch = gtk_timeout_add(1000, Ginterface_Plugins_Output_Update, NULL);
}

void Ginterface_Plugins_ToolBar(GtkItemFactory *host_menu) {
   static GtkItemFactoryEntry action_menu[] = {
      { "/Quit", "q", Ginterface_Plugins_Callback, 'q', "<StockItem>", GTK_STOCK_QUIT },
      { "/Action", "h", NULL, 0, "<Branch>", NULL},
      { "/Action/Toggle Plugin Output", "p", Ginterface_Plugins_Callback, 'p', "<StockItem>", GTK_STOCK_JUMP_TO},
      { "/Help", NULL, NULL, 0, "<LastBranch>", NULL},
      { "/Help/_Manual", NULL, Ginterface_Manual, 0, "<StockItem>", GTK_STOCK_HELP},
      { "/Help/_About", NULL, Ginterface_About, 0, "<StockItem>", GTK_STOCK_DIALOG_INFO}
   };
   gint num_items = 0;

   DEBUG_MSG("Ginterface_ToolBar");

   gtk_item_factory_delete_item(host_menu, "/Quit");
   gtk_item_factory_delete_item(host_menu, "/Action");
   gtk_item_factory_delete_item(host_menu, "/Help");

   num_items = sizeof (action_menu) / sizeof (action_menu[0]);
   gtk_item_factory_create_items (host_menu, num_items, action_menu, NULL);
}

void Ginterface_Plugins_Callback(gpointer data, guint action, GtkWidget *widget) {
   switch (action) {
      case 'H':
      case 'h':
         {
            static char *help[] = {
               "[qQ][F10] - quit",
               "[return]  - if external: run the selected plug in",
               "          - if hooking : activate/deactivate it",
               "[hH]      - this help screen",
               NULL};
            Ginterface_HelpWindow(help);
         }
         break;
      case 'p':
         Ginterface_Plugins_Output();
         break;
      case 'q':
      case 'Q':
         {
            char answer;
            answer = Ginterface_PopUpQ("Do you really want to exit?");
            if ((answer == 'y') || (answer == 'Y'))
               Ginterface_WExit("They are safe!!  for now... ");
         }
         break;
   }
}

void Ginterface_Plugins_Stop(void) {
   int i;
   DEBUG_MSG("Ginterface_Plugins_Stop");

   gtk_timeout_remove(id_watch);

   for(i=0; i< number_of_plugins; i++)    // activate the plugins
      if (Plugins_Array[i].status == 'A')
         Plugin_SetActivation(Plugins_Array[i].name, 1);
      else if (Plugins_Array[i].status == ' ')
         Plugin_SetActivation(Plugins_Array[i].name, 0);

   gtk_container_remove(GTK_CONTAINER (mainbox[5]), p_scrolled);
   g_free(p_list);
   p_list = NULL;
   p_scrolled = NULL;
}

#endif   // PERMIT_PLUGINS

/* EOF */


// vim:ts=3:expandtab

