/*
    ettercap -- gtk interface for splash screen

    Copyright (C) 2001  ALoR <alor@users.sourceforge.net>, NaGA <crwm@freemail.it>
    GTK+ 2.0 interface by daten <daten@dnetc.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>
#include <ec_gtk_icons.h>

#include <ec_main.h>

/* display a splash screen with the given image file */
void Ginterface_Splash_Init(void);

/* append a message to it */
void Ginterface_Splash_Append(gchar *mesg);

/* append a message with a progress bar 
 *
 * see http://developer.gnome.org/doc/API/2.0/gtk/gtkprogressbar.html
 * or
 * src/ec_parser.c for example
 */
void Ginterface_Splash_Progress(gchar *mesg);

/* close the splash screen */
void Ginterface_Splash_Close(void);

GtkWidget *dialog, *label, *image, *pbar;

void Ginterface_Splash_Init(void) {
   GdkPixbuf *pixbuf;
        
   dialog = gtk_dialog_new_with_buttons ("ettercap", NULL, GTK_DIALOG_MODAL, NULL);
   gtk_container_set_border_width (GTK_CONTAINER (dialog), 10);
   gtk_window_set_default_size(GTK_WINDOW (dialog), 300, 300);
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
   gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);

   pixbuf = gdk_pixbuf_new_from_inline (-1, ettercap_logo, FALSE, NULL);
   image = gtk_image_new_from_pixbuf(pixbuf);
   gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), image, FALSE, FALSE, 0);
   gtk_widget_show(image);

   gtk_widget_show (dialog);

   while (gtk_events_pending ())
      gtk_main_iteration ();
}

/* this makes a new label in the splash window */
void Ginterface_Splash_Append(gchar *mesg) {
   label = gtk_label_new (mesg);
   gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), label, FALSE, FALSE, 0);
   gtk_widget_show(label);

   /* this loop brings the interface up-to-date without returning to gtk_main() */
   while (gtk_events_pending ())
      gtk_main_iteration ();
}

/* this makes a new label and progress bar in the splash window */
void Ginterface_Splash_Progress(gchar *mesg) {
   label = gtk_label_new (mesg);
   gtk_misc_set_alignment (GTK_MISC (label), 0, 1);
   gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), label, FALSE, FALSE, 0);
   gtk_widget_show(label);

   /* this progress bar will need to be updated somewhere with */
   /* gtk_progress_bar_set_fraction() or something similar     */
   pbar = gtk_progress_bar_new ();
   gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), pbar, FALSE, FALSE, 0);
   gtk_widget_show(pbar);

   while (gtk_events_pending ())
      gtk_main_iteration ();
}

void Ginterface_Splash_Close(void) {
   gtk_widget_destroy (dialog);

   while (gtk_events_pending ())
      gtk_main_iteration ();
}

/* EOF */

// vim:ts=3:expandtab
